function Set-FileExtension {
    gci -Recurse *.nfo | % { Move-Item $_.FullName ($_.FullName -replace '.nfo', '.txt') }
}

function Sync-Directory {
    [CmdletBinding(SupportsShouldProcess = $true)]
    param(
        [Parameter(Position = 0, Mandatory, ValueFromPipeline)][ValidateScript( {Test-Path $_ -PathType Container})][string] $source,
        [Parameter(Position = 1, Mandatory, ValueFromPipelineByPropertyName)][ValidateScript( {Test-Path $_ -PathType Container})][string] $destination
    )

    $sourceFiles = Get-ChildItem $source -Recurse | Select-Object -ExpandProperty Name
    $destinationFiles = Get-ChildItem $destination -Recurse | Select-Object -ExpandProperty Name

    Compare-Object $sourceFiles $destinationFiles | ForEach-Object {
        if ($_.SideIndicator -eq '=>') {
            $file = Get-ChildItem $destination -Filter $($_.InputObject) -Recurse

            if ($PSCmdlet.ShouldProcess($file.FullName, "Remove-Item")) {
                Write-Host "Removing '$($_.InputObject)' from '$source'"
                Remove-Item $file.FullName
            }
        }
        elseif ($_.SideIndicator -eq '<=') {
            # Find the file to copy, it can be in a sub-directory
            $file = Get-ChildItem $source -Filter $($_.InputObject) -Recurse

            # Get the file source directory from the file, by removing the source path
            $fileDirectory = $file.DirectoryName.Replace($source, '')
            $fileDirectoryDestination = Join-Path $destination $fileDirectory
            $fileFullDestination = Join-Path $fileDirectoryDestination $_.InputObject

            if (-not (Test-Path $fileDirectoryDestination)) {
                if ($PSCmdlet.ShouldProcess($fileDirectoryDestination, "New-Item")) {
                    # Create the file directory at the destination
                    New-Item -ItemType Directory $fileDirectoryDestination
                }
            }

            if ($PSCmdlet.ShouldProcess($file.FullName, "Copy-Item")) {
                Write-Host "Copying '$($_.InputObject)' to '$fileFullDestination'"

                # Copy the file to the destination preserving the source directory
                Copy-Item $file.FullName $fileFullDestination
            }
        }
    }
}