function Start-Logging {
    $transcript = Start-Transcript

    Start-Process (Get-FileTail $transcript -Wait)

    while ($true) {
        if ($Host.UI.RawUI.KeyAvailable -and (3 -eq [int]$Host.UI.RawUI.ReadKey("AllowCtrlC,IncludeKeyUp,NoEcho").Character)) {
            Start-Transcript | Out-Null

            Remove-Item $transcript

            break
        }
    }
}

function New-Credentials {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0)][string] $user = 'boyan',
        [Parameter(Position = 1)][string] $value = 'zIwIcUO#O8qFZg*l!x34'
    )

    $password = ConvertTo-SecureString $value -AsPlainText -Force
    $credentials = New-Object System.Management.Automation.PSCredential($user, $password)

    return $credentials
}

function New-RemoteSession {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0, Mandatory)][string] $server
    )

    $credentials = New-Credentials

    Enter-PSSession $server -Credential $credentials
}

function Connect-BoxStarterShare {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0)][string] $server = 'win10-vm'
    )

    $credentials = New-Credentials
    $trustedHostExists = Get-TrustedHost | Where-Object { $_ -eq $server }

    if (-not $trustedHostExists) {
        Set-TrustedHost $server
    }

    if (!(Test-Path "\\$server\BoxStarter")) {
        $driveLetter = (Get-ChildItem function:[d-z]: -n | Where-Object {!(test-path $_)} | Random) -replace ':', ''

        New-PSDrive -Name $driveLetter -PSProvider FileSystem `
            -Root "\\$server\BoxStarter" `
            -Credential $credentials `
            -Scope Global | Out-Null

        if (!(Test-Path "\\$server\BoxStarter")) {
            Write-Host "Mapped to $driveLetter`:\"
        }
    }
}

function Enable-Remoting {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0)][string] $remoteHost = 'beast',
        [Parameter(Position = 1)][string] $sharePath = 'C:\BoxStarter'
    )

    $localhost = $env:computerName.toLower()

    Set-NetConnectionProfile -InterfaceIndex (Get-NetConnectionProfile).InterfaceIndex -NetworkCategory Private

    Enable-PSRemoting -Force

    Set-Item wsman:\localhost\client\trustedhosts $remoteHost -Confirm:$false

    $shareName = Split-Path -Leaf $sharePath

    if (-not (Get-SmbShare $shareName)) {
        New-Item -ItemType Directory -Path $sharePath -Force | Out-Null
        New-SmbShare -Name $shareName -Path $sharePath -FullAccess 'Everyone' | Out-Null
    }

    Restart-Service WinRM

    Test-WsMan $localhost
}

function Get-Checksum {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0, Mandatory)][string] $file
    )

    $checksum = (Get-FileHash $file).Hash
    Set-Clipboard $checksum

    return $checksum
}

function Add-Path {
    foreach ($level in "Machine", "User") {
        [Environment]::GetEnvironmentVariables($level).GetEnumerator() | % {
            # For Path variables, append the new values, if they're not already in there
            if ($_.Name -match 'Path$') {
                $_.Value = ($((Get-Content "Env:$($_.Name)") + ";$($_.Value)") -split ';' | Select -unique) -join ';'
            }
            $_
        } | Set-Content -Path { "Env:$($_.Name)" }
    }
}

function Get-CeritifcateRequest {
    [CmdletBinding()]
    param(
        [Parameter(Position = 0, Mandatory)][string] $hostHeader,
        [Parameter(Position = 1)][string] $outputDir = 'C:\Requests'
    )

    $date = (Get-Date).ToString('ddMMyyyy')
    New-Item -ItemType Directory $outputDir -ErrorAction SilentlyContinue

    $requestFile = Join-Path $outputDir "Certificate-Request-$hostHeader-$date.req"

    $infFile = @"
    [NewRequest]`r
    Subject = "CN=$hostHeader"`r
    KeySpec = 1
    KeyLength = 2048
    Exportable = TRUE`r
    RequestType = CMC`r
"@

    $finalInfFile = Join-Path $outputDir "Certificate-Request-$hostHeader-$date.inf"
    New-Item $finalInfFile -Type File -Value $infFile
    & certreq -new $finalInfFile $requestFile

    Write-Host "Certificate Request Generated at $finalInfFile..."
}