[CmdletBinding(SupportsShouldProcess = $true)]
Param(
    [Parameter(Mandatory = $false)][ValidateScript( {Test-Path (Resolve-Path $_)})][string] $dropboxDir = (Join-Path $PSScriptRoot '..\..\..' -Resolve)
)

# Resolve in case we got a relative path
$dropboxDir = Resolve-Path $dropboxDir

$info = "Processing Directory: $dropboxDir"
if ($PSCmdlet.ShouldProcess("$info")) {
    Write-Host $info
}

$files = @()
$duplicates = @{}

# Find all the "conflicted" files Dropbox creates on sync
gci -R -Force -File $dropboxDir -Filter '*conflicted*' | `
 % { `
    # Remove the (pcname conflicted...) from the file
    # therefore getting the original file name
    $name = $_.FullName -replace '\s\(.*\)', ''

    # If the file is not already in the duplicates hashtable
    if (!$duplicates.ContainsKey($name)) {
        # Create an empty array to store all duplicates
        $duplicates[$name] = @()
    }

    # Add the duplicate file to the hashtbale
    # where the hashtable key is the original file name
    $duplicates[$name] += $_
}

# Sort the duplicates keys, where the key is the original file name
# and for each key, itterate over the duplicate files
$duplicates.Keys | Sort-Object | % {
    # For each aray of duplicate files
    # Sort the array descending by last write time, and take the first item, the most recent one
    $duplicates.$_ | Sort-Object LastWriteTime -Descending | Select-Object -First 1 | % {
        # Remove the (pcname conflicted...) from the file
        $name = $_.Name -replace '\s\(.*\)', ''

        # Recreate the path to the original file name
        $newFullName = $_.FullName -replace '\s\(.*\)', ''

        # Also need the current directory
        $currentDir = Split-Path -Parent $_.FullName

        if (Test-Path $_.FullName) {
            $info = "$($_.FullName) --> $newFullName"

            if ($PSCmdlet.ShouldProcess("$info")) {
                # Overwrite the original file with the most recent duplicate
                Write-Host $info
                Move-Item $_.FullName $newFullName -Force
            }
        }

        # Find and delete any other files
        # that are duplicates (conflicted) of the orginal file
        gci $currentDir -Filter "*$name*conflicted*" | % {
            $info = "Remove-Item: $_"

            if ($PSCmdlet.ShouldProcess("$info")) {
                Remove-Item "$_"
            }
        }
    }
}