[CmdletBinding(SupportsShouldProcess = $true)]
Param(
)

$commonStarMenuDir = 'C:\ProgramData\Microsoft\Windows\Start Menu\Programs'
$userStartMenDir = Join-Path $env:AppData 'Microsoft\Windows\Start Menu\Programs'
$tempStartMenuDir = Join-Path $env:Temp 'TempStartMenu'

if (Test-Path $tempStartMenuDir) {
    Get-ChildItem $tempStartMenuDir -Recurse | Remove-Item -Recurse -Force
}
else {
    New-Item -ItemType Directory $tempStartMenuDir | Out-Null
}

$startMenuFolders = @{
    'Accessories'          = @()
    'Administrative Tools' = @()
    'Backup'               = @()
    'Internet'             = @()
    'Media'                = @()
    'Productivity'         = @()
    'Programming'          = @(
        'Microsoft Azure'
        'Microsoft SQL Server 2016'
        'Microsoft SQL Server vNext CTP2.0'
        'Visual Studio 2017'
    )
    'Startup'              = @()
    'Utilities'            = @()
}

$startMenuShortcuts = @{
    'Backup'       = @(
        'Backblaze'
        'Dropbox'
    )
    'Internet'     = @(
        'All-in-One Messenger'
        'Google Chrome'
        'Signal Private Messenger'
        'Slack'
    )
    'Media'        = @(
        'ACDsee'
        'HandBrake'
        'MetaX'
        'paint.net'
        'Spotify'
    )
    'Productivity' = @(
        'Evernote'
        'EditPad Pro 7'
        'Excel 2016'
        'Google Drive'
        'Power BI Desktop'
        'OneNote 2016'
        'PowerPoint 2016'
        'Publisher 2016'
        'Visio 2016'
        'Word 2016'
    )
    'Programming'  = @(
        'Blend for Visual Studio 2017'
        'ConEmu (x64)'
        'Git Extensions'
        'KDiff3'
        'JetBrains DataGrip.*'
        'JetBrains WebStorm.*'
        'Microsoft Web Platform Installer'
        'Microsoft SQL Server Management Studio 17'
        'SQL Server Profiler 17'
        'Node.js command prompt'
        'Node.js'
        'Postman'
        'Visual Studio Code'
        'Visual Studio 2017'
        'Visual Studio 2015'
        'Visual Studio Installer'
        'Node.js'
        'VMware PowerCLI'
    )
    'Utilities'    = @(
        '7-Zip File Manager'
        'CCleaner'
        'Defraggler'
        'LogFusion'
        'NTLite'
        'Rufus'
        'TeamViewer 12'
        'UpdateGenerator'
        'UpdateInstaller'
        'WinDirStat'
        'WinPcap'
    )
    'Startup'      = @(
        'Launchy'
    )
}

$startMenuFolders.Keys | ForEach-Object {
    New-Item -ItemType Directory (Join-Path $tempStartMenuDir $_) | Out-Null
}

# For each of the directories in the original start menu
$originalStartMenuDirs = Get-ChildItem -Directory $commonStarMenuDir -Recurse
$originalStartMenuDirs += Get-ChildItem -Directory $userStartMenDir -Recurse
$originalStartMenuDirs | ForEach-Object {
    $originalDir = $_

    # If the directory is in the defined list of start menu directories,
    # copy it to the appropriate category directory
    $startMenuFolders.GetEnumerator() | Where-Object { $_.Value -contains $originalDir } | ForEach-Object {
        Copy-Item $originalDir.FullName (Join-Path $tempStartMenuDir $_.Key) -Recurse -Force
    }
}

# For each of the file shortucts in the original start menu
$originalStartMenuShortcuts = Get-ChildItem -File $commonStarMenuDir -Recurse
$originalStartMenuShortcuts += Get-ChildItem -File $userStartMenDir -Recurse
$originalStartMenuShortcuts | ForEach-Object {
    $originalShortcut = $_

    # For each shortuct defined in the list of shortucts
    $startMenuShortcuts.GetEnumerator() | ForEach-Object {
        # The key is the current sub menu directory
        $subMenuDir = $_.Key

        # If the list of values contains exact match for the original shotcut
        $_.Value | Where-Object { $originalShortcut.BaseName -match "^$_$" } | ForEach-Object {
            # Copy the original shortuct to the appropriate category sub menu directory
            Copy-Item $originalShortcut.FullName (Join-Path $tempStartMenuDir $subMenuDir)
        }
    }

    # For each of the directories defined in the organized start menu,
    # if the parent of directory name of the original shortcut matches the categorized directory name
    $startMenuFolders.GetEnumerator() | Where-Object { (Split-Path -Leaf $originalShortcut.DirectoryName) -eq $_.Key } | ForEach-Object {
        # Copy the original shortuct to the appropriate category sub menu directory
        Copy-Item $originalShortcut.FullName (Join-Path $tempStartMenuDir $_.Key) -Recurse -Force
    }
}

# Remove all items in the current common and user start menu directories
Get-ChildItem $commonStarMenuDir | Remove-Item -Recurse -Force
Get-ChildItem $userStartMenDir | Remove-Item -Recurse -Force

if ($PSCmdlet.ShouldProcess("$tempStartMenuDir --> $commonStarMenuDir", "Copy-Item")) {
    # Copy organized start menu to the common start menu directory
    Get-ChildItem $tempStartMenuDir | Copy-Item -Destination $commonStarMenuDir -Recurse -Force
}

# Remove the temporary storage directory
if ($PSCmdlet.ShouldProcess($tempStartMenuDir, "Remove-Item")) {
    Remove-Item $tempStartMenuDir -Recurse -Force
}